#!/usr/bin/python3
import sys
import os
import configparser
import sqlite3
import subprocess
import getpass

def refresh_uu_config(new_config_file,old_config_file):
    config_new=configparser.ConfigParser(allow_no_value=True)
    config_new.optionxform = str
    config_old=configparser.ConfigParser(allow_no_value=True)
    config_old.optionxform = str
    config_new.read(new_config_file)
    config_old.read(old_config_file)
    for section in config_new.sections():
        if not config_old.has_section(section):
            config_old.add_section(section)
            print("new section %s added"%(section))  
        for option in config_new.options(section):
            if not config_old.has_option(section,option):
                value = config_new[section][option]
                config_old.set(section,option,value)
                print("new option:%s %s"%(option,value))
    with open(old_config_file,'w') as f:
        config_old.write(f)

def setconfvalue(section,key,value):
    cmd="kconf2 set %s %s %s"%(str(section),str(key),str(value))
    print(cmd)
    ret=subprocess.run(cmd,shell=True)
    return ret.returncode
    

def refresh_frontend_yaml():
    config_file = "/var/lib/unattended-upgrades/unattended-upgrades-policy.conf"
    db_file="/var/lib/kylin-system-updater/kylin-system-updater.db"
    experience_config_file="/var/lib/kylin-software-properties/config/updateExperience.conf"
    server_address_config_file="/var/lib/kylin-software-properties/template/kylin-source-update.conf"
    if (os.path.exists(db_file)):
        conn = sqlite3.connect(db_file)
        cursor = conn.cursor()
        cursor.execute('SELECT auto_check FROM display')
        dat = cursor.fetchone()
        auto_check=dat[0]
        print("auto_check:%s"%auto_check)
        setconfvalue("kylin-update-frontend.notification","enable",auto_check)
        cursor.execute('SELECT download_limit FROM display')
        dat = cursor.fetchone()
        download_limit=dat[0]
        print("download_limit:%s"%download_limit)
        setconfvalue("kylin-update-frontend.downloadlimit","enable",download_limit)
        cursor.execute('SELECT download_limit_value FROM display')
        dat = cursor.fetchone()
        download_limit_value=dat[0]
        print("download_limit_value:%s"%download_limit_value)
        setconfvalue("kylin-update-frontend.downloadlimit","value",download_limit_value)
        cursor.close()
        conn.close()     
    if (os.path.exists(config_file)):
        config=configparser.ConfigParser(allow_no_value=True)
        config.optionxform = str
        config.read(config_file)
        dat=config['autoUpgradePolicy']['autoUpgradeState']
        auto_download="true"
        if(dat=="on"):
            auto_download="true" 
        elif(dat=="off"):
            auto_download="false"
        print("auto_download:%s"%auto_download)
        setconfvalue("kylin-update-frontend.autodownload","enable",auto_download)              
        dat=config['autoUpgradePolicy']['updateDays']
        print("update days:%s"%dat)
        setconfvalue("kylin-update-frontend.downloadsettings","period",dat)
        worktime="true"       
        user=os.getlogin()
        if os.path.exists("/home/%s/.config/durationcheckbox"%user):
            worktime="true"
        else:
            worktime="false"
        # dat=config['autoUpgradePolicy']['workTime']
        # worktime="true"
        # if(dat=="on"):
        #     worktime="true" 
        # elif(dat=="off"):
        #     worktime="false"
        print("work time:%s"%worktime)
        setconfvalue("kylin-update-frontend.downloadsettings","designated",worktime)
        dat=config['autoUpgradePolicy']['downloadTime']
        print("download time:%s"%dat)
        setconfvalue("kylin-update-frontend.downloadsettings","designatedtime",dat)
    if (os.path.exists(experience_config_file)):
        config=configparser.ConfigParser(allow_no_value=True)
        config.optionxform = str          
        config.read(experience_config_file)
        dat=config['Experience']['isUpdateExperience']
        print("betaversion:%s"%dat)
        setconfvalue("kylin-update-frontend.betaversion","enable",dat) 
    if (os.path.exists(server_address_config_file)):
        config=configparser.ConfigParser(allow_no_value=True)
        config.optionxform = str
        config.read(server_address_config_file)
        dat=config['Server']['Address']
        print("server address:%s"%dat)
        setconfvalue("kylin-update-frontend.serversettings","address",dat) 
        dat=config['Server']['Port']
        print("server port:%s"%dat)
        setconfvalue("kylin-update-frontend.serversettings","port",dat) 

if __name__ == "__main__":
    new_config_file = "/usr/share/kylin-system-updater/unattended-upgrades-policy.conf"
    old_config_file = "/var/lib/unattended-upgrades/unattended-upgrades-policy.conf"
    db_file="/var/lib/kylin-system-updater/kylin-system-updater.db"
    print("argv length:%d"%len(sys.argv))
    if len(sys.argv) >=4:     
        new_config_file=sys.argv[1]
        old_config_file=sys.argv[2]
        db_file=sys.argv[3]
    try:
        refresh_uu_config(new_config_file,old_config_file)
        refreshyamlflag="/var/lib/conf2-refresh-flag"
        if(os.path.exists(refreshyamlflag)):
            pass
        else:
            print("refresh conf2 db")
            refresh_frontend_yaml()
            with open(refreshyamlflag,'w') as f:
                f.write("already refreshed")
    except Exception as e:
        print(e)
        os._exit(0)


