import time
import platform
import sqlite3
import subprocess
import re
import logging
import os
from datetime import datetime
from copy import deepcopy
import json
import gzip

DBname = '/var/lib/kylin-system-updater/InstallDebs_TimeDB.db'
# DBname = '../../data/InstallDebs_TimeDB.db'
# 由于没有numpy库, 手写了一个二维数组归一化方法
def normalize_by_column(matrix:list):
    # 翻转二维数组
    def rotate_by_column(matrix):
        rotated = list(zip(*matrix))
        return [list(row) for row in rotated]
    
    # 归一化一维数组
    def normalize_rows(matrix):
        return [
            # [float(element) / sum(row) for element in row]
            [float(element) / max(row) for element in row ]
            for row in matrix
        ]
    new_matrix = rotate_by_column(matrix)
    results = normalize_rows(new_matrix)
    results = rotate_by_column(results)
    return results


class calTimeManager():
    def __init__(self) -> None:
        pass
    def CalInstallTimeByDebsList_min(self, debList:list, debug=False) -> float:
        # python3在某些场景下，编译运行时可能认为该赋值语句会导致全局变量被声明成局部变量，导致报错
        # if debug:
        #     global DBname
        #     DBname = '../../data/InstallDebs_TimeDB.db'
        result = 1
        installTimeSum = 0
        unpackTimeSum = 0
        configTimeSum = 0
        triggerTimeSum = 0
        # Process local dpkg.log
        logging.info("Start collecting local installation time information")
        try:
            callocaltimehelper = cal_time()
            callocaltimehelper.run("/var/log/","dpkg.log")
            logging.info("Collected local installation time information SUCCESS")
        except Exception as e:
            logging.error("Collected local installation time information ERROR")
            logging.error(e)
        
        # Load database
        logging.info("Start loading database")
        try:
            # Init Database
            global DBname
            conn = sqlite3.connect(DBname)
            table_name = "local_info"
            logging.info(f"The database is {table_name}")
            cursor = conn.cursor()

            if self._check_ifDBhasTableforarchitecture(cursor, table_name) is False:
                raise Exception(f'Init database error, missing {table_name} table')
            # 获取基准信息
            datas, not_found_items = self._getAllInfoForDBTable(cursor, debList, table_name)
            conn.close()
            for (name,installTime, unpackTime,configTime,triggerTime) in datas:
                installTimeSum += installTime
                unpackTimeSum += unpackTime
                configTimeSum += configTime
                triggerTimeSum += triggerTime
            logging.info(f"local dpkg {len(datas)} packages install time is {installTimeSum}s")
            range_time = installTimeSum / len(datas)
            result += self._calnoinfodebsinstalltime(not_found_items, range_time)
            result += installTimeSum

            # result += len(debList)/5
            self._write_datas_2_local(datas, '/var/lib/kylin-system-updater/json/installTime_info.txt')
        except Exception as e:
            logging.error("Loaded database ERROR")
            logging.error(e)
            return 1

        # 换算成分钟
        result = int(result / 60)
        if result == 0:
            result = 1
        return int(result)

    def _write_datas_2_local(self, datas, file_path):
        # 确保 file_path 的目录存在
        dir_path = os.path.dirname(file_path)
        if dir_path and not os.path.exists(dir_path):
            os.makedirs(dir_path)
        
        with open(file_path, 'w', encoding='utf-8') as file:
            for item in datas:
                file.write(str(item) + '\n')

    def _calnoinfodebsinstalltime(self,not_found_items, Unit_time=1):
        '''
        判断是否数据库中没有的包为内核包，如果是，则加上固定估计时间
        内核包编译安装时间大概为10分钟，经过检测
        '''
        result = 0
        isLinuxKernelDebs = False
        for items in not_found_items:
            if "linux" in items:
                isLinuxKernelDebs = True
                break
        if isLinuxKernelDebs is True:
            result += 600
        
        # 如果还有不存在的数据，则按照10个包40秒的时间判断
        logging.info(f"Unit_time is {Unit_time}, pkg num is {len(not_found_items)}")
        if len(not_found_items) > 10:
            result += len(not_found_items) * Unit_time
        return result
            
    def _check_ifDBhasTableforarchitecture(self, cursor, table_name):
        try:
            cursor.execute("SELECT name FROM sqlite_master WHERE type='table';")
            tables = cursor.fetchall()
            logging.info(tables)
        except Exception as e:
            logging.error(e)
            return False
        if (table_name,) in tables:
            return True
        else:
            return False

    def _getAllInfoForDBTable(self, cursor, debList, table_name):
        all_data = None
        rectifyDebList = []
        for deb in debList:
            deb = deb.replace("#auto","")
            rectifyDebList.append(deb)
        try:
            placeholders = ','.join('?' for _ in rectifyDebList)
            query = f"SELECT * FROM {table_name} WHERE package_name IN ({placeholders})"
            # Execute the query with the list of package names
            cursor.execute(query, rectifyDebList)
            all_data = cursor.fetchall()
            # 提取找到的值
            found_items = [row[0] for row in all_data]
            # 列表中未找到的行
            not_found_items = list(set(rectifyDebList) - set(found_items))
            logging.info(f"Packages that are missing from the database:  {not_found_items}")
            logging.info(f"The quantity that is missing from the database:{len(not_found_items)}")
        except Exception as e:
            logging.error(e)
            return None
        # logging.info(all_data)
        return all_data, not_found_items
class cal_time():
    KetWordsDict = {
        'FORMAT' : "%(asctime)s [%(levelname)s]: %(message)s",
        # 标识dpkg日志的开始行，包括新装、升级、
        'STARTUP_INSTALL_FLAG' : "startup archives unpack",
        'STARTUP_INSTALL_FLAG2' : "startup archives install",

        # 解压阶段的所有标识
        'INSTALL_UNPACK_FLAG' : "install ",
        'UPGRADE_UNPACK_FLAG' : "upgrade ",
        'SINGLE_UNPACK_FINISHED_FLAG' : "status unpacked ",
        'SINGLE_UNPACK_FLAG2' : 'status not-installed',
        'ALL_UNPACK_FINISHED_FLAG' : "startup packages configure",

        # 配置阶段的所有标识
        'START_CONFIGURE' : "configure ",
        'SINGLE_CONFIGURE_FINISHED' : "status installed",
        'SINGLE_CONFIGURE_FINISHED_OTHER' : "status triggers-awaited",
        'ALL_CONFIGURE_FINISHED' : "trigproc ",

        # 触发器阶段的所有标识
        'START_TRIGGER' : "trigproc ",
        'SINGLE_TRIGGER_FINISHED' : "status installed"
    }
    def __init__(self) -> None:
        self.pkgname_instime_dict = {}
        pass
    

    def run(self, log_path, file_name):

        log_files = self._find_dpkg_log_files(log_path, file_name)
        log_lines = []
        for log_file in log_files:
            file_lines = self._read_dpkg_log(log_file)
            log_lines.extend(file_lines)
        logging.info(f"local dpkg.log length is: {len(log_lines)}")
        pkglist = self._getpkgnamelist(log_lines)
        logging.info(f"local dpkg information has debs: {len(set(pkglist))}")
        self.pkgname_instime_dict = self._inittimetable(pkglist)
        self._caltime(log_lines)
        # logging.info(self.pkgname_instime_dict)
        self._calinstalltime()
        self._SaveInDB()
    
    def _find_dpkg_log_files(self, directory, file_name):
        """查找目录下所有以 file_name 开头的文件"""
        return sorted([os.path.join(directory, f) for f in os.listdir(directory) if f.startswith(file_name)], reverse=True)

    def _read_dpkg_log(self, file_path):
        """读取 dpkg 日志文件内容，支持普通文件和 gzip 压缩文件"""
        open_func = gzip.open if file_path.endswith('.gz') else open
        lines = []
        with open_func(file_path, 'rt', encoding='utf-8') as f:
            for line in f:
                lines.append(line.strip())
        return lines
    
    def _find_new_line(self, log_lines, data, time):
        def _filter_by_time(line, data, time):
            strings = line.split(' ')
            if strings[0] >= data and strings[1] >= time:
                return True
            else:
                return False
        # 对dpkg的日志进行检索开始的行，并裁剪开始行之前的数据
        for line in log_lines[:]:  # 使用 char_list[:] 复制原列表，避免在遍历时修改原列表
            if _filter_by_time(line, data, time) is False:
                log_lines.remove(line)
            else:
                break

        for line in log_lines[:]: 
            string = line.split(' ')
            if string[2] != "install" and string[2] != "upgrade":
                log_lines.remove(line)
            else:
                break
        return log_lines
            
    def _cla_unpack_configure_time(self, log_lines):
        unpack_time = {}
        while log_lines:
            start_time_line = log_lines.pop(0)
            pkgs_data = start_time_line.split()
            start_pkg_name = pkgs_data[3].split(":")[0]
            start_time = pkgs_data[1]

            end_time_line = log_lines.pop(0)
            pkgs_data = end_time_line.split()
            end_pkg_name = pkgs_data[4].split(":")[0]
            end_time = pkgs_data[1]
            
            if start_pkg_name != end_pkg_name:
                logging.info(start_time_line)
                logging.info("数据异常，解压阶段的顺序出现异常，开始时间的包名:{0}和结束时间包名不一致:{1}".format(start_pkg_name, end_pkg_name))
                # sys.exit(1)
            
            unpack_time[start_pkg_name] = self._time_diff(start_time,end_time)
        return unpack_time[start_pkg_name]

    def _time_diff(self, start_time,end_time, fmt = None):
        if fmt is None:
            fmt = "%H:%M:%S.%f"
        t1 = datetime.strptime(end_time, fmt)
        t2 = datetime.strptime(start_time, fmt)
        time_diff = t1 - t2
        seconds = time_diff.total_seconds()
            # 统计过程可能会出现时间时区变化
        if seconds >=28800:
            seconds -= 28800
        if seconds == 0:
            seconds += 1
        return (seconds)

    def _getunpacktime(self, log_lines):
        unpack_stage = []
        lines = deepcopy(log_lines)
        stage_type = 'start'
        log_data = lines[0]
        while lines:
            # 提取上一行log的时间戳
            last_row_time = log_data.split(' ')[0] + " " + log_data.split(' ')[1]
            log_data = lines.pop(0)

            # 查看当前是什么阶段
            if self.KetWordsDict['START_CONFIGURE'] in log_data:
                stage_type = 'configing'

            if self.KetWordsDict['INSTALL_UNPACK_FLAG'] in log_data:
                stage_type = 'end'
                # 替换时间戳
                log_data = log_data.replace(log_data.split(' ')[0] + " " + log_data.split(' ')[1], last_row_time)
                unpack_stage.append(log_data)

            elif self.KetWordsDict['UPGRADE_UNPACK_FLAG'] in log_data:
                stage_type = 'end'
                # 替换时间戳
                log_data = log_data.replace(log_data.split(' ')[0] + " " + log_data.split(' ')[1], last_row_time)
                unpack_stage.append(log_data)
                # 直接弹出三个数据，过滤掉升级数据存在两个相同status unpacked的日志输出 
                lines.pop(0)
                lines.pop(0)
                lines.pop(0)
            elif (self.KetWordsDict['SINGLE_UNPACK_FINISHED_FLAG'] in log_data or self.KetWordsDict['SINGLE_UNPACK_FLAG2'] in log_data) and stage_type == 'end':
                stage_type = 'start'
                unpack_stage.append(log_data)
        return self._cla_unpack_configure_time(unpack_stage)
    
    def _getpkgnamelist(self, log_lines):
        result = []
        for line in log_lines:
            strings = line.split(' ')
            if strings[2] == "install"  or strings[2] == "upgrade" or strings[2] == "configure" or strings[2] == "trigproc":
                list = strings[3].split(':')
                result.append(list[0])
        return set(result)

    def _inittimetable(self, pkglist):
        result = {}
        for pkg in pkglist:
            result[pkg] = {"installtime":0,"unpacktime":0,"configtime":0,"trigtime":0}
        return result

    def _caltime(self, log_lines):
        lines = deepcopy(log_lines)
        findend = False
        proc = ""
        pkgname = ""
        starttime = ""
        endtime = ""
        while lines:
            line = lines.pop(0)
            string = line.split(' ')
            if string[2] == "install":
                proc = "unpacktime"
                findend = True
                pkgname = (string[3].split(':'))[0]
                starttime = string[1]
                
            elif string[2] == "upgrade":
                proc = "unpacktime"
                findend = True
                pkgname = (string[3].split(':'))[0]
                starttime = string[1]
                
            elif string[2] == "configure":
                proc = "configtime"
                findend = True
                pkgname = (string[3].split(':'))[0]
                starttime = string[1]
                
            elif string[2] == "trigproc":
                proc = "trigtime"
                findend = True
                pkgname = (string[3].split(':'))[0]
                starttime = string[1]

            if findend is True:
                if proc == "unpacktime" and "status unpacked" in line:
                    if pkgname == string[4].split(':')[0]:
                        endtime = string[1]
                        self.pkgname_instime_dict[pkgname][proc] = self._time_diff(starttime, endtime, fmt="%H:%M:%S")
                        findend = False
                        # logging.info(pkgname,self.pkgname_instime_dict[pkgname][proc])
                        proc = ""
                        pkgname = ""
                        starttime = ""
                        endtime = ""
                    else:
                        # logging.info("查找unpack结尾", pkgname)
                        continue
                
                elif proc == "configtime" and \
                    ("status installed" in line or "status triggers-awaited" in line or "trigproc" in line):
                    if pkgname == string[4].split(':')[0]:
                        endtime = string[1]
                        self.pkgname_instime_dict[pkgname][proc] = self._time_diff(starttime, endtime, fmt="%H:%M:%S")
                        findend = False
                        # logging.info(pkgname, proc, self.pkgname_instime_dict[pkgname][proc])
                        proc = ""
                        pkgname = ""
                        starttime = ""
                        endtime = ""
                    else:
                        # logging.info("查找config结尾", pkgname)
                        continue
                elif proc == "trigtime" and "status installed" in line:
                    if pkgname == string[4].split(':')[0]:
                        endtime = string[1]
                        self.pkgname_instime_dict[pkgname][proc] = self._time_diff(starttime, endtime, fmt="%H:%M:%S")
                        findend = False
                        # logging.info(pkgname, proc, self.pkgname_instime_dict[pkgname][proc])
                        proc = ""
                        pkgname = ""
                        starttime = ""
                        endtime = ""
                    else:
                        # logging.info("查找trig结尾", pkgname)
                        continue
    def _calinstalltime(self):
        result = 0
        num = 0
        numlist = []
        for pkg in self.pkgname_instime_dict:
            if self.pkgname_instime_dict[pkg]["configtime"] <= 1 and \
                self.pkgname_instime_dict[pkg]["trigtime"]<= 1 and \
                self.pkgname_instime_dict[pkg]["unpacktime"]<=1:
                 self.pkgname_instime_dict[pkg]["installtime"] = 1
            else:
                self.pkgname_instime_dict[pkg]["installtime"] += \
                            self.pkgname_instime_dict[pkg]["configtime"]+ \
                            self.pkgname_instime_dict[pkg]["trigtime"] + \
                            self.pkgname_instime_dict[pkg]["unpacktime"]
            if self.pkgname_instime_dict[pkg]["installtime"] > 1200:
                self.pkgname_instime_dict[pkg]["installtime"] = 600
                num+=1
                numlist.append(pkg)
            
            result += self.pkgname_instime_dict[pkg]["installtime"]
        logging.info(f"all pkg install time is {result}")
        logging.info(numlist)
    
    def _SaveInDB(self):
        self.table_name = 'local_info'
        global DBname
        conn = sqlite3.connect(DBname)
        cursor = conn.cursor()
        for deb in self.pkgname_instime_dict:
            try:
                check_query = f'''
    SELECT * FROM {self.table_name}
    WHERE package_name='{deb}'
    '''
                cursor.execute(check_query)
                result = cursor.fetchall()
                if len(result) == 0:
                    insert_query=f'''
                                INSERT INTO {self.table_name} (package_name, installtime, unpacktime, configtime, trigtime)
                                VALUES ('{deb}',    '{self.pkgname_instime_dict[deb]['installtime']}', \
                                                    '{self.pkgname_instime_dict[deb]['unpacktime']}', \
                                                    '{self.pkgname_instime_dict[deb]['configtime']}', \
                                                    '{self.pkgname_instime_dict[deb]['trigtime']}')
                                '''
                    # logging.info(insert_query)
                    cursor.execute(insert_query)
                else:
                    insert_query=f'''
                                UPDATE {self.table_name} SET
                                installtime = '{self.pkgname_instime_dict[deb]['installtime']}',
                                unpacktime = '{self.pkgname_instime_dict[deb]['unpacktime']}',
                                configtime = '{self.pkgname_instime_dict[deb]['configtime']}',
                                trigtime = '{self.pkgname_instime_dict[deb]['trigtime']}'
                                WHERE package_name = '{deb}';
                                '''
                    # logging.info(insert_query)
                    cursor.execute(insert_query)
                    pass
            except Exception as e:
                # logging.info(e)
                pass
        conn.commit()
        conn.close()