import dbus
import logging
from SystemUpdater.Core.GenericConfigParser import Gconfig
from gettext import gettext as _
from SystemUpdater.Core.errors import *
from SystemUpdater.Core.enums import *

def check_device_power(bus,pkgs_mun):
        def _battry_limit(power_type,pkgs_mun):
            # 最小电量限制
            MINI_POWER = 20
            # 最大电量限制
            MAX_POWER = 50

            # 最大软件包数量阈值
            MAX_PKGS_MUN = 500

            # 插入电源的标识为   插上电源充电 1 或者 插上电源充电，充满4 
            plug_ac = (power_type == 1 or power_type == 4)
            if not plug_ac:
                return 50

            # 基准数
            radix = (MAX_POWER - MINI_POWER) / MAX_PKGS_MUN

            # 根据软件包数量 计算电量要求
            requires = int(MINI_POWER + radix * pkgs_mun)
            if requires > MAX_POWER:
                requires = MAX_POWER
            return requires

        power_percentage = 0
        requires = 0
        try:
            # 支持调试模式 进行关闭
            skip_check_list = Gconfig().get("skip_check_list",[])
            if ERROR_DEVICE_LOW_BATTERY in skip_check_list:
                return
        
            obj = bus.get_object("org.freedesktop.UPower","/org/freedesktop/UPower/devices/DisplayDevice")
            interface = dbus.Interface(obj, "org.freedesktop.DBus.Properties")

            # 获取供电方式
            power_type = int(interface.Get("org.freedesktop.UPower.Device","State",timeout=0.5))
            
            logging.info("Check: Currently powered state:%d",power_type)
            # 非电池方式直接退出检查
            # 1. 台式机为0   2. 插上电源充电 1   3. 未插入电源，用电池2 4. 插上电源充电，充满4 
            # if power_type == 0 or power_type == 1 or power_type == 4:
            if power_type == 2:
                # 电池供电方式
                power_percentage = int(interface.Get("org.freedesktop.UPower.Device","Percentage",timeout=0.5))
                requires = _battry_limit(power_type,pkgs_mun)
                logging.info("Check: Now device power is percentage:%d and requires:%d...",power_percentage,requires)

        except Exception as e:
            logging.error(str(e))

        if power_percentage != 0 and power_percentage < requires:
            logging.error("Not enough device power to shutdown install")
            raise UpdateBaseError(ERROR_DEVICE_LOW_BATTERY)  
        return 