#!/usr/bin/env python
# -*- Mode: Python; indent-tabs-mode: nil; tab-width: 4; coding: utf-8 -*-
# (c) 2005-2012 Canonical, GPL
# (C) 2008-2009 Sebastian Heinlein <devel@glatzor.de>

from __future__ import print_function

import aptdaemon.client as client
import aptdaemon.errors as errors
import aptdaemon.enums as enums

from aptdaemon import client, errors
from defer import inline_callbacks
from SystemUpdater.Core.errors import *
from SystemUpdater.Core.enums import *
from aptdaemon.enums import (EXIT_SUCCESS,
                             EXIT_FAILED,
                             get_status_string_from_enum)

from SystemUpdater.backendnext import InstallBackend
from importlib import reload
from gettext import gettext as _


class InstallBackendAptdaemon(InstallBackend):
    """Makes use of aptdaemon to refresh the cache and to install updates."""

    def __init__(self, window_main, action):
        InstallBackend.__init__(self, window_main, action)
        self.window_main = window_main
        reload(client)
        reload(errors)
        reload(enums)
        self.client = client.AptClient()
        self.progress = 0

    @inline_callbacks
    def update(self):
        """刷新包cache"""
        try:
            trans = yield self.client.update_cache(defer=True)
            # 注册回调函数 接收更新的状态
            yield self._show_transaction(trans, self.ACTION_UPDATE)
        except errors.NotAuthorizedError:
            self._action_done(self.ACTION_UPDATE,
                            authorized=False, success=False,
                            error_string='', error_desc='')
        except Exception:
            self._action_done(self.ACTION_UPDATE,
                            is_cancelled=False, success=False,
                            error_string='', error_desc='')
            raise

    @inline_callbacks
    def _show_transaction(self, trans, action):
        trans.connect("finished", self._on_finished, action)
        trans.connect("progress-changed", self._on_progress_changed,action)
        trans.connect("status-changed", self._on_status_changed,action)
        yield trans.run()

    def _on_progress_changed(self, trans,progress,action):
        #不要101这种未知状态
        if progress == 101:
            return
        if progress > self.progress:
            self.progress = progress

    def _on_status_changed(self, trans, status,action):
        self.window_main.dbusController.UpdateCacheStatusChanged(self.progress,get_status_string_from_enum(status))

    def _on_finished(self, trans, status, action):
        error_code = ''
        error_desc = ''
        if status == EXIT_FAILED and trans.error and trans.error.code:
            error_code = get_error_num_from_enum(trans.error.code)
            error_desc = get_error_string_from_enum(trans.error.code) + " "+ get_error_description_from_enum(trans.error.code)
        is_success = (status == EXIT_SUCCESS)
        self.backup_serverid()
        self.window_main.dbusController.UpdateCacheFinishedNext(is_success,[''],error_code,error_desc)

    def backup_serverid(self):
        import os
        import shutil

        SERVERIDPATH = "/var/lib/kylin-software-properties/config/serverID.conf"
        SERVERIDPATH1 = "/var/lib/kylin-software-properties/config/updateID.conf"
        BACKUPPATH   = "/var/lib/kylin-system-updater/transID/"

        try:
            if not os.path.exists(BACKUPPATH):
                os.makedirs(BACKUPPATH, mode=0o755, exist_ok=True)

            if os.path.isfile(SERVERIDPATH):
                dst = shutil.copy(SERVERIDPATH, BACKUPPATH)

            if os.path.isfile(SERVERIDPATH1):
                dst = shutil.copy(SERVERIDPATH1, BACKUPPATH)

        except Exception as e:
            pass