#!/bin/bash

# set -x 
#bash -c "$(curl -fksSL https://172.20.191.209/wangsong/analyse-problem/-/raw/master/analyse-update-error.sh)"

#bash -c "$(curl -fksSL https://gitee.com/newwangsong/analyse-update/raw/master/analyse-update-error.sh)"

ERROR_DESC_FILE="/tmp/update-details.log"
RECORD_PKGS_VERSION=(kylin-system-updater kylin-activation kylin-update-frontend libkylin-activation 
                    ukui-notification-daemon ukui-session-manager)

TRUE=true
FALSE=false

APT_TERM_LOG="/var/log/apt/term.log"
UPDATER_LOG_PATH="/var/log/kylin-system-updater/kylin-system-updater.log.1"
PUSH_CONTENT="/var/lib/kylin-software-properties/template/important.list"
# 检查文件是否存在
if ! [ -e "$UPDATER_LOG_PATH" ]; then
    UPDATER_LOG_PATH="/var/log/kylin-system-updater/kylin-system-updater.log.1.1.gz"
fi

OFFLINE_SOURCE_PATH="/media/OfflineSource"

OFFLINE_SOURCE_FLAG=$FALSE

ERROR_DESC=""
SOURCE_TYPE=""
PUSH_UPDATE_PATCH=""
SOURCES_LIST=""

# 执行目录定向到/tmp下
cd /tmp
ANALYSE_LOG_DIR="/tmp/analyse-log"
mkdir -p $ANALYSE_LOG_DIR >/dev/null 2>&1 || true 
rm -rf updaterlog >/dev/null 2>&1 || true
#建立收集的log目录
mkdir updaterlog >/dev/null 2>&1 || true

# 检查是否带有参数
if [[ "$1" == "--output" ]]; then
    OUTPUT_FILE=$TRUE
else
    OUTPUT_FILE=$FALSE
fi

rm $ERROR_DESC_FILE >/dev/null 2>&1 || true 

BOLD="$(tput bold 2>/dev/null || printf '')"
GREY="$(tput setaf 0 2>/dev/null || printf '')"
UNDERLINE="$(tput smul 2>/dev/null || printf '')"
RED="$(tput setaf 1 2>/dev/null || printf '')"
GREEN="$(tput setaf 2 2>/dev/null || printf '')"
YELLOW="$(tput setaf 3 2>/dev/null || printf '')"
BLUE="$(tput setaf 4 2>/dev/null || printf '')"
MAGENTA="$(tput setaf 5 2>/dev/null || printf '')"
NO_COLOR="$(tput sgr0 2>/dev/null || printf '')"

info() {
    printf '%s\n\n' "${BOLD}${GREY}>${NO_COLOR} $*" | tee -a $ERROR_DESC_FILE
}

warn() {
    printf '%s\n' "${YELLOW}! $*${NO_COLOR}" | tee -a $ERROR_DESC_FILE
}

error() {
    printf '%s\n\n' "${BOLD}> $*${NO_COLOR}" | tee -a $ERROR_DESC_FILE
}

completed() {
  printf '%s\n' "${GREEN}✓${NO_COLOR} $*" | tee -a $ERROR_DESC_FILE
}

# -----------------------------------------------------------------------------
# Make sure everything really stops when CTRL+C is pressed
# -----------------------------------------------------------------------------

fn_terminate_script() {
	exit 1
}
trap 'fn_terminate_script' SIGINT


# 检查推送内容
if [ -f "/var/lib/kylin-software-properties/template/important.list" ]; then
    if [ -s "/var/lib/kylin-software-properties/template/important.list" ]; then
        PUSH_UPDATE_PATCH=$(cat /var/lib/kylin-software-properties/template/important.list)
    else
        PUSH_UPDATE_PATCH=""
    fi
else
    PUSH_UPDATE_PATCH=""
fi

# 检查源信息
if [ -f "/etc/kylin-system-updater/apt_private.conf" ]; then
    SOURCES_LIST="/var/lib/kylin-system-updater/sources.list"
    APT_RUN="apt -c /etc/kylin-system-updater/apt_private.conf"
else
    SOURCES_LIST="/etc/apt/sources.list"
    APT_RUN="apt"
fi

# 检查设备是否属于内部试用 切换不同的提示方案
if [ -f "/var/lib/kylin-appreciate-backup/LICENSE" ]; then
    INTERNAL_TRIAL=$TRUE
else
    INTERNAL_TRIAL=$FALSE
fi

if [ -f "/etc/kylin-system-updater/apt_private.conf" ]; then
    AB_SOURCES=$TRUE
else
    AB_SOURCES=$FALSE
fi

if [ -f $SOURCES_LIST ]; then
    # 检查是否使用离线源
    if grep -q "deb file:" "$SOURCES_LIST"; then
        SOURCE_TYPE="离线源更新"
        OFFLINE_SOURCE_FLAG=$TRUE
    else
        SOURCE_TYPE="在线源更新"
    fi
else
    SOURCE_TYPE="在线源更新"
fi

arg_prompt(){
    info "${BOLD}更新方式${NO_COLOR}：  ${GREEN}${SOURCE_TYPE}${NO_COLOR}"
    info "${BOLD}更新内容${NO_COLOR}：  ${GREEN}${PUSH_UPDATE_PATCH}${NO_COLOR}"
    info "${BOLD}更新源${NO_COLOR}：    ${GREEN}${SOURCES_LIST}${NO_COLOR}"
    info "${BOLD}更新错误${NO_COLOR}：  ${GREEN}${ERROR_DESC}${NO_COLOR}"
    error "${BOLD}错误描述${NO_COLOR}：  ${BOLD}${RED}$1${NO_COLOR}"
    error "${BOLD}解决方案${NO_COLOR}：  ${BOLD}${RED}$2${NO_COLOR}"
    if ! [ -z "$3" ]; then
        info "${BOLD}其他描述${NO_COLOR}：  ${GREEN}$3${NO_COLOR}"
    fi

    report_updater_log
    exit 0
}

report_updater_log() {
    date >> updaterlog/base-info

    # 记录包版本
    for pkg in "${RECORD_PKGS_VERSION[@]}"; do
        dpkg -l | grep $pkg >> updaterlog/base-info
    done

    info "${BOLD}正在导出系统更新日志，请稍后...${NO_COLOR}"

    # 收集当前系统配置
    cp /var/lib/kylin-system-updater/sources.list updaterlog/private-sources.list >/dev/null 2>&1 || true 
    cp /etc/apt/sources.list updaterlog/system-sources.list >/dev/null 2>&1 || true 
    cp -r /etc/apt/apt.conf.d updaterlog >/dev/null 2>&1 || true 
    cp -r /usr/share/kylin-update-desktop-config updaterlog >/dev/null 2>&1 || true 
    cp -r /var/lib/kylin-system-updater/json updaterlog/upgrade-data >/dev/null 2>&1 || true 
    cp -r /var/cache/kylin-system-updater/update_available_data.json updaterlog/upgrade-data >/dev/null 2>&1 || true 
    cp -r /var/cache/kylin-system-updater/install-debs-md5.json updaterlog/upgrade-data >/dev/null 2>&1 || true 
    cp -r /var/lib/dpkg/status updaterlog >/dev/null 2>&1 || true 

    #收集系统更新的日志
    cp -r /var/log/apt updaterlog >/dev/null 2>&1 || true 
    cp -r /var/log/dpkg.log updaterlog >/dev/null 2>&1 || true 
    cp -r /var/log/kylin-unattended-upgrades updaterlog >/dev/null 2>&1 || true 
    cp -r ~/.config/kylin-background-upgrade updaterlog >/dev/null 2>&1 || true 
    cp -r /var/log/kylin-system-updater/ updaterlog >/dev/null 2>&1 || true 
    cp -r /var/log/syslog updaterlog >/dev/null 2>&1 || true 
    cp -r  /var/cache/kylin-system-updater/*.bak updaterlog >/dev/null 2>&1 || true 
    cp -r /var/lib/kylin-system-updater/InstallDebs_TimeDB.db updaterlog/upgrade-data >/dev/null 2>&1 || true 

    #激活
    mkdir -p updaterlog/kylin-activation
    cp -r ~/.log/kylin-activation/ updaterlog/kylin-activation >/dev/null 2>&1 || true 
    cp /etc/LICENSE updaterlog/kylin-activation >/dev/null 2>&1 || true 
    cp /etc/.kyinfo updaterlog/kylin-activation >/dev/null 2>&1 || true 

    #备份还原日志
    cp -r /var/log/backup.log updaterlog >/dev/null 2>&1 || true 

    #P2P
    cp -r /var/log/apt-p2p.log updaterlog >/dev/null 2>&1 || true 

    # 源更新管理器
    cp -r /var/log/kylin-software-properties/ updaterlog >/dev/null 2>&1 || true 

    #收集前端日志
    cp -r ~/.log/kylin-update-frontend-notifysend.log updaterlog >/dev/null 2>&1 || true
    cp -r ~/.log/ukui-control-center.log updaterlog >/dev/null 2>&1 || true 
    cp -r ~/.log/ukui-notification-daemon.log updaterlog >/dev/null 2>&1 || true
    cp -r ~/.config/ukui-session/ updaterlog >/dev/null 2>&1 || true
    cp -r /tmp/kylin-updateresult-notify.log updaterlog >/dev/null 2>&1 || true

    cp -r $ANALYSE_LOG_DIR/* updaterlog >/dev/null 2>&1 || true 
    cp -r $ERROR_DESC_FILE updaterlog >/dev/null 2>&1 || true 

    outputName="$(date +%m-%d-%H-%M-%S)-updaterLog.tar.gz"
    #将所有的日志进行打包
    tar -czvf updaterLog.tar.gz updaterlog >/dev/null

    #删除收集的日志目录
    rm -rf updaterlog

    #系统升级收集bug日志使用
    if [ $(id -u) -eq 0 ]; then
        mv updaterLog.tar.gz /tmp/$outputName
        info "${BOLD}请将日志文件提交到禅道,输出位置：${UNDERLINE}/tmp/$outputName${NO_COLOR}"
    else
        #将文件存储到桌面
        if [ ! -d ~/桌面 ]; then
            mv updaterLog.tar.gz ~/Desktop/$outputName
            info "${BOLD}请将桌面下日志文件提交到禅道,输出位置：${UNDERLINE}~/Desktop/$outputName${NO_COLOR}"
        else
            mv updaterLog.tar.gz ~/桌面/$outputName
            info "${BOLD}请将桌面下日志文件提交到禅道,输出位置：${UNDERLINE}~/桌面/$outputName${NO_COLOR}"
        fi
    fi
}

report_apt_lists() {
    if ! [ -z "$1" ]; then
        if [ "$AB_SOURCES" = "$TRUE" ]; then
            cp -r /var/lib/apt/lists updaterlog >/dev/null 2>&1 || true
        else
            cp -r /var/lib/kylin-system-updater/lists updaterlog >/dev/null 2>&1 || true
        fi
    fi
}


fn_apt_download() {
    if [ -f "/etc/kylin-system-updater/apt_private.conf" ]; then
        apt download $1 -c /etc/kylin-system-updater/apt_private.conf
	else
        apt download $1
    fi
}

fn_apt_install() {
    if [ -f "/etc/kylin-system-updater/apt_private.conf" ]; then
        sudo apt install --simulate $1 -c /etc/kylin-system-updater/apt_private.conf
	else
        sudo apt install --simulate $1
    fi
}

fn_update_problem() {
    error_desc="未检查到错误或错误目前不支持检查。"
    solution="请联系系统更新研发，并上传日志。"

    arg_prompt $error_desc $solution
}

fn_install_failed() {
    ERROR_MSG_CN="在处理时有错误发生："
    ERROR_MSG_EN="Errors were encountered while processing:"
    LOG_START="Log started:"

    append_dsc="详细错误日志查看/var/log/apt/term.log，简要错误描述："
    INSTALL_FAILED_SAFE="验证失败，拒绝安装"
    INSTALL_FAILED_SAFE_OTHER="未签名，拒绝安装"

    INSTALL_FAILED_SCRIPT="子进程返回错误状态"
    INSTALL_FAILED_SCRIPT_EN="script subprocess returned error"

    INSTALL_FAILED_INCLUDE="同时被包含于软件包"
    NSTALL_FAILED_INCLUDE_EN="which is also in package"

    INSTALL_FAILED_SAFE_CONTROL="检测到未认证程序试图执行"
    NSTALL_FAILED_SAFE_CONTROL_EN="检测到未认证程序试图执行"

    SAFE_WHITELIST_FAILED="in the prevent uninstallation of whitelist!"

    term_msg=$(tac "$APT_TERM_LOG" | awk "/$ERROR_MSG_CN|$ERROR_MSG_EN/ {flag=1} flag {print} /Log started:/ {exit}" | tac)

    if [[ $term_msg == *"$INSTALL_FAILED_SAFE"* || $term_msg == *"$INSTALL_FAILED_SAFE_OTHER"* ]]; then        
        error_type="软件包验签失败或者未签名"
        solution="请联系安全事业部研发人员并且指派相应bug"
        error_desc=$(echo "$term_msg" | grep -E "$INSTALL_FAILED_SAFE|$NSTALL_FAILED_SAFE_CONTROL_EN")
    elif [[ $term_msg == *"$INSTALL_FAILED_SAFE_CONTROL"* || $term_msg == *"$INSTALL_FAILED_SAFE_OTHER"* ]]; then        
        error_type="终端日志中，出现选择未认证程序试图执行的选项。"
        solution="请联系安全事业部研发人员并且指派相应bug"
        error_desc=$(echo "$term_msg" | grep -E "$INSTALL_FAILED_SAFE_CONTROL|$NSTALL_FAILED_SAFE_CONTROL_EN")
    # 安装前后脚本执行错误
    elif [[ $term_msg == *"$INSTALL_FAILED_SCRIPT"* || $term_msg == *"$INSTALL_FAILED_SCRIPT_EN"* ]]; then
        # 获取安装错误的包
        error_pkg=$(echo "$term_msg" | grep -E "$INSTALL_FAILED_SCRIPT|$INSTALL_FAILED_SCRIPT_EN" | awk '{print $2}')
        error_type="软件包安装前后脚本执行错误"
        error_pkg_desc="错误软件包："$error_pkg""
        maintainer=$(zcat /usr/share/doc/$error_pkg/changelog.Debian.gz | grep -m 1 '^ -- ' | awk -F' -- ' '{print $2}' | awk '{print $1, $2}')
        solution="请蓝信联系上述的包维护人("$maintainer")，并提交相应bug"
        error_desc=$(echo "$term_msg" | grep -E -B 5 "$INSTALL_FAILED_SCRIPT|$INSTALL_FAILED_SCRIPT_EN")
    elif [[ $term_msg == *"$INSTALL_FAILED_INCLUDE"* || $term_msg == *"$NSTALL_FAILED_INCLUDE_EN"* ]]; then
        # 获取安装错误的包
        error_pkg=$(echo "$term_msg" | grep -E "$INSTALL_FAILED_INCLUDE|$NSTALL_FAILED_INCLUDE_EN" | awk -F'软件包 ' '{print $2}' | awk '{print $1}')
        error_type="两个软件包包含相同文件"
        error_pkg_desc="错误软件包："$error_pkg""
        maintainer=$(zcat /usr/share/doc/$error_pkg/changelog.Debian.gz | grep -m 1 '^ -- ' | awk -F' -- ' '{print $2}' | awk '{print $1, $2}')
        solution="请蓝信联系上述的软件包维护人("$maintainer")，并提交相应bug"
        error_desc=$(echo "$term_msg" | grep -E -B 1 "$INSTALL_FAILED_INCLUDE|$NSTALL_FAILED_INCLUDE_EN")
    elif [[ $term_msg == *"$SAFE_WHITELIST_FAILED"* ]]; then
        error_type="软件包在禁止卸载白名单"
        solution="请联系安全事业部研发人员并且指派相应bug"
        error_desc=$(echo "$term_msg" | grep "$SAFE_WHITELIST_FAILED")
    else
        error_type="未审计到安装错误信息或系统更新存在异常。"
        solution="请联系系统更新研发，并上传日志。"
    fi

    case $1 in
        easy)
            echo "$error_type $error_pkg_desc"
            exit 0
            ;;
        details)
            echo "$error_desc"
            exit 0
            ;;
        *)
            arg_prompt "$error_type $error_pkg_desc" "$solution" "$append_dsc$error_desc"
    esac
    
}

fn_check_update_source() {
    # 检查是否采用A/B源机制
    # 检查离线源deb file:
    
    if [ "$OFFLINE_SOURCE_FLAG" = "$TRUE" ]; then
        if [ -e "$OFFLINE_SOURCE_PATH" ]; then
            if [ -n "$(ls -A "$OFFLINE_SOURCE_PATH")" ]; then
                ppp=ppp
            else
                arg_prompt "离线源被卸载，可能由于关闭控制面板或者其他原因。" "需要重新挂载离线源，再进行更新"                
            fi
        else
            arg_prompt "离线源被卸载，可能由于关闭控制面板或者其他原因。" "需要重新挂载离线源，再进行更新"
        fi
    fi

    info "${BOLD}进行apt update测试，等待5分钟左右请稍后...${NO_COLOR}"
    if [ -f "/etc/kylin-system-updater/apt_private.conf" ]; then
        other_desc="修复后执行: apt update -c /etc/kylin-system-updater/apt_private.conf 进行检查。"
        append_desc=$(LANG= sudo timeout 5m apt update -c /etc/kylin-system-updater/apt_private.conf 2>&1)
	else
        other_desc="修复后执行: apt update 进行检查是否正常。"
        append_desc=$(LANG= sudo timeout 5m apt update 2>&1)
    fi

    if [ $? -eq 0 ]; then
        fn_update_problem
    elif [ $? -eq 124 ]; then
        error_desc="检查更新 apt update 执行超时"
        solution="1、请检查网络是否正常； 2、检查是否开启限速"
    else
        if [[ $append_desc == *"invalid for another"* ]]; then
            error_desc="检查更新apt update失败，未校准时间"
            solution="请校准时间再进行系统更新。$other_desc"
        elif [[ $append_desc == *"is not signed"* ]]; then
            error_desc="检查更新apt update失败，存在未签名的软件源"
            solution="请联系版本组校对推送的源信息。$other_desc"
        elif [[ $append_desc == *"sources.list"* ]]; then
            error_desc="apt update失败，sources.list 源格式异常。"            
            if [ -f "/etc/kylin-system-updater/apt_private.conf" ]; then
                solution="请检查推送的源格式是否正常，$other_desc"
            else
                solution="请检查推送的源格式是否正常，$other_desc"
            fi

        else
            error_desc="软件源更新apt update更新失败"
            solution="检查：1.使用的源是否正确，2.检查host的配置。若其他设备都存在问题，则联系版本组检查源,$other_desc"
        fi
    fi

    arg_prompt "$error_desc" "$solution" "$append_desc"
}

fn_check_latest() {
    # 检查推送文件是否存在
cat > /tmp/check-sources.py <<EOF
#!/bin/python3
import configparser
 
import subprocess
import json
import psutil
import requests
import json

import os



class Colors:
    HEADER = '\033[95m'
    OKBLUE = '\033[94m'
    OKGREEN = '\033[92m'
    WARNING = '\033[93m'
    FAIL = '\033[91m'
    ENDC = '\033[0m'
    BOLD = '\033[1m'
    UNDERLINE = '\033[4m'
 



class check:
    def __init__(self):
        self.address = ""
        self.port = ""
        self.mac = ""
        self.ip = ""
        self.kysn = ""
        self.systemVersion=""
        self.productLine = ""
        

    def checkReallyUpgrade(self):
        
        pass

    def getSystemVersion(self):
        # 先获取buildID
        kybuilderID = ""
        try:
            with open("/etc/kylin-build", "r") as file:
                content = file.read()
                if "buildid:" in content:
                    parts = content.split("buildid:")
                    kybuilderID = parts[1].strip().replace("\n", "").replace(" ", "")
        except FileNotFoundError:
            print("/etc/kylin-build 文件不存在")
        if kybuilderID == "":
            try:
                with open("/etc/os-release", "r") as file:
                    content = file.read()
                    if "\n" in content:
                        for line in content.split("\n"):
                            if "KYLIN_RELEASE_ID" in line:
                                m_systemVersion = line.replace("KYLIN_RELEASE_ID=", "").strip().replace("\"", "")
                                break
                        else:
                            m_systemVersion = ""  # 如果循环结束没有找到，设置为空字符串
            except FileNotFoundError:
                print("os-release 打开失败")
                m_systemVersion = ""
            
            if m_systemVersion == "":
                print(f"读取文件获取的版本号 {m_systemVersion} {m_patchVersion}")
                if not m_systemVersion:  # 如果m_systemVersion为空
                    m_systemVersion = "2107"
        else:
            config = configparser.ConfigParser()
            try:
                config.read("/etc/kylin-version/kylin-system-version.conf")
                system = config.get("SYSTEM", "os_version")
                patch = config.get("SYSTEM", "update_version")
                if system != patch and patch != "":
                    m_systemVersion = patch
                else:
                    m_systemVersion = kybuilderID
            except (configparser.NoSectionError, configparser.NoOptionError, FileNotFoundError):
                m_systemVersion = kybuilderID

        if "*" in m_systemVersion:
            m_systemVersion = m_systemVersion.replace("*", "")
        self.systemVersion = m_systemVersion

        print(f"{Colors.OKBLUE} 您的系统版本号为: {Colors.ENDC}",self.systemVersion)


# 获取ukui-control-center的版本
    def get_pkg_version_from_pkg_name(self,pkg_name):
        try:
            result = subprocess.run(['dpkg', '-l', pkg_name], capture_output=True, text=True)
            lines = result.stdout.splitlines()
            for line in lines:
                if pkg_name in line:
                    version = line.split()
                    return version[2]
        except FileNotFoundError:
            print("dpkg 命令未找到")
            return ""

    def getProductLine(self):
        m_productLine = ""
        UKCVersion = self.get_pkg_version_from_pkg_name("ukui-control-center")

        # 获取内核版本
        process = subprocess.run(['uname', '-r'], capture_output=True, text=True)
        kernelVersion = process.stdout.strip()
        if len(kernelVersion) < 6:
            kernelVersion += "0.0.0.0"

        # 获取架构
        process = subprocess.run(['arch'], capture_output=True, text=True)
        arch = process.stdout.strip()
        m_arch = arch

        # 根据架构和UKCVersion设置产品线
        if arch in ["sw_64", "loongarch64", "mips64"]:
            if UKCVersion.startswith("3.24"):
                m_productLine = "xc-2303u2"
            else:
                m_productLine = "xc"

        # 读取/proc/cpuinfo并检查特定的模型名称
        try:
            with open("/proc/cpuinfo", "r") as file:
                tmp = file.read()
        except FileNotFoundError:
            tmp = ""

        list_of_lines = tmp.split("\n")
        for str_line in list_of_lines:
            if "model name" in str_line.lower():
                if arch.lower() == "aarch64":
                    if any(keyword in str_line.lower() for keyword in ["phytium", "kunpeng", "ft-2000", "d2000", "ft2000"]) or kernelVersion.startswith("5.4.18"):
                        if UKCVersion.startswith("3.24"):
                            m_productLine = "xc-2303u2"
                        else:
                            m_productLine = "xc"
                        # 同样，这里可能是一个返回点或其他逻辑
                        break  # 找到匹配项后退出循环（如果需要）
                elif arch.lower() == "x86_64":
                    if any(keyword in str_line.lower() for keyword in ["zhaoxin", "hygon"]) or kernelVersion.startswith("5.4.18"):
                        if UKCVersion.startswith("3.24"):
                            m_productLine = "xc-2303u2"
                        else:
                            m_productLine = "xc"
                        break  # 找到匹配项后退出循环（如果需要）
                    if "intel" in str_line.lower() or "amd" in str_line.lower():
                        if kernelVersion.startswith("5.4"):
                            if UKCVersion.startswith("3.24"):
                                m_productLine = "xc-2303u2"
                            else:
                                m_productLine = "xc"
                            break  # 找到匹配项后退出循环（如果需要）
                        elif kernelVersion.startswith("5.10"):
                            if UKCVersion.startswith("3.24"):
                                m_productLine = "hwe-2303u2"
                            else:
                                m_productLine = "hwe"
                            # 同样，这里可能是一个返回点或其他逻辑
                            break  # 找到匹配项后退出循环（如果需要）
                        elif kernelVersion.startswith("5.15"):
                            if UKCVersion.startswith("3.24"):
                                m_productLine = "hwe-pp-2303u2"
                            else:
                                m_productLine = "hwe-pp"
                            # 同样，这里可能是一个返回点或其他逻辑
                            break  # 找到匹配项后退出循环（如果需要）
                # 处理包含"hardware"的特定情况
                if "hardware" in str_line.lower():
                    if arch.lower() == "aarch64":
                        # 处理Kirin 990和特定内核版本的情况
                        if ("kirin" in str_line.lower() and "990" in str_line.lower()) or kernelVersion.startswith("4.19.71"):
                            if UKCVersion.startswith("3.24"):
                                m_productLine = "wayland-990-2303u2"
                            else:
                                m_productLine = "wayland-990"

                            break  # 找到匹配项后退出循环（如果需要）
                        # 处理Kirin 9006c和特定内核版本的情况
                        elif ("kirin" in str_line.lower() and "9006c" in str_line.lower()) or kernelVersion.startswith("5.4.96"):
                            if UKCVersion.startswith("3.24"):
                                m_productLine = "wayland-9006c-2303u2"
                            else:
                                m_productLine = "wayland-9006c"

                            break  # 找到匹配项后退出循环（如果需要
        self.productLine = m_productLine
    def getServerAddress(self):
        #获取服务器地址
        # 创建一个ConfigParser对象
        try:
            config=configparser.RawConfigParser()
            config.read("/var/lib/kylin-software-properties/template/kylin-source-update.conf",encoding="utf-8")
            self.address = config.get("Server","address")
            self.port = config.get("Server","port")
            print("检测出的服务器地址:",self.address+":"+self.port)
            print(f"{Colors.OKBLUE}请检查服务器地址是否正确，如果是公网应为http://archive1.kylinos.cn:59546,私有化的为自行填写的地址.如果不对，请修改/var/lib/kylin-software-properties/template/kylin-source-update.conf文件中address以及port字段{Colors.ENDC}")
        except:
            print(f"{Colors.FAIL}/var/lib/kylin-software-properties/template/kylin-source-update.conf 文件内容格式有问题{Colors.ENDC}")
        
        
    def getPcInfo(self):
        # 获取序列号
        try:
            exec = "cat /etc/.kyinfo |grep '^key' | awk -F '=' '{print $2}'"
            process = os.popen(exec).read()
            self.kysn = process.strip()
        except:
            print(f"{Colors.FAIL}获取授权失败，请检查授权(/etc/.kyinfo)是否有问题{Colors.ENDC}")
            exit()
        #获取mac地址 
        command = "ip route show default | awk '{print $5}'"
        process = subprocess.Popen(command, shell=True, stdout=subprocess.PIPE, stderr=subprocess.STDOUT, universal_newlines=True)
        # 打印输出
        for line in process.stdout:
            with open(("/sys/class/net/"+line.strip()+"/address"), 'r') as file:
                content = file.read()
                if self.mac == "":
                    self.mac = content.strip()
                else:
                    self.mac = self.mac +"@" + content.strip()
                
                command = "LANG=C ifconfig "+ line.strip() + " | awk '/inet /{ print $2 }'"
                result = subprocess.run(command, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE, text=True)
                if self.ip == "":
                        self.ip = result.stdout.strip()
                else:
                    self.ip = self.ip +"@" + result.stdout.strip()
        process.wait()

        print("检测到您的服务序列号为:" + self.kysn +"  mac地址:" + self.mac + "  IP地址:" + self.ip + f"{Colors.FAIL}   请检查是否与您申请升级的机器信息一致{Colors.ENDC}")

    def checkPushResult(self):
        # params = {"mac":self.mac,"ip":self.ip,"epqtSn":"","serverSn":self.kysn,"activationCode":"","lastRequestAt":"","cpu":"","registerCode":"","enableExperiencePlan":""}
        url = self.address + ":" + self.port + "/transaction/list?serverSn="+ self.kysn
        response = requests.get(url)
        # url = "http://10.42.189.250:59546/transaction/list?serverSn=46878214"
        response = requests.get(url)
        data = json.loads(response.text)
        if len(data['data']['records']) == 0:
            print(f"{Colors.FAIL}推送的事务数为0，请校验上报需要推送的服务序列号与本机是否一致，如一致，请联系管理员检测推送策略是否正确{Colors.ENDC}")
            exit()
        isHaveUpgrade = False
        for item in data['data']['records']:
            for node in item['nodes']:
                systemVersionList = []
                productLineList = []
                #判断条件是否满足
                for condition in node['conditions']:
                    if condition['category']['code'] == 1:
                        systemVersionList.append(condition['value'])
                    elif condition['category']['code'] == 4:
                        productLineList.append(condition['value'])
                if (self.systemVersion in systemVersionList or len(systemVersionList) == 0) and (self.productLine in productLineList or len(productLineList) == 0):
                    isHaveUpgrade = True
                    index = item['nodes'].index(node)
                    for index in range(len(item['nodes'])):
                        if item['nodes'][index]['category']['code'] == 2:
                            if os.path.exists("/var/lib/kylin-software-properties/data/upgrade.log"):
                                with open('/var/lib/kylin-software-properties/data/upgrade.log', 'r') as file:
                                    content = file.read()
                                    if item['nodes'][index]['uuid'] in content:
                                        sources = ""
                                        important = item['nodes'][index]['configInfo']['importantPackages']
                                        for source in item['nodes'][index]['configInfo']['sources']:
                                            sources  = sources + source['link'] + "\n"
                                            sources = sources.strip()
                                        print(f"{Colors.OKBLUE}此更新已经完成，具体更新的原地址为：\n{Colors.ENDC}", sources, "具体推送内容为:",important)
                                        continue
                                    else:
                                        sources = ""
                                        important = item['nodes'][index]['configInfo']['importantPackages']
                                        for source in item['nodes'][index]['configInfo']['sources']:
                                            sources  = sources + source['link'] + "\n"
                                        print(f"{Colors.OKBLUE}此次更新具体更新的原地址为：\n{Colors.ENDC}", sources, "具体推送内容为:",important)
                                        print(f"{Colors.OKBLUE}请校验推送的原地址和推送列表是否与预期一致，质量更新推送列表为kylin-update-desktop-quality，系统升级为kylin-update-desktop-system，如果不一致\
                                            ，请校验上报需要推送的服务序列号与本机是否一致，如一致，请联系管理员检测推送策略是否正确{Colors.ENDC}")
                                        break
                            else:
                                sources = ""
                                important = item['nodes'][index]['configInfo']['importantPackages']
                                for source in item['nodes'][index]['configInfo']['sources']:
                                    sources  = sources + source['link'] + "\n"
                                print(f"{Colors.OKBLUE}此次更新具体更新的原地址为：\n{Colors.ENDC}", sources, "具体推送内容为:",important)
                                print(f"{Colors.OKBLUE}请校验推送的原地址和推送列表是否与预期一致，质量更新推送列表为kylin-update-desktop-quality，系统升级为kylin-update-desktop-system，如果不一致\
                                    ，请校验上报需要推送的服务序列号与本机是否一致，如一致，请联系管理员检测推送策略是否正确{Colors.ENDC}")
                                break
                    if isHaveUpgrade:
                        break
                if isHaveUpgrade:
                    break
            if isHaveUpgrade is True:
                break
        if isHaveUpgrade is False:
            print(f"{Colors.FAIL}推送不满足条件，您的系统版本:{Colors.ENDC}",self.systemVersion, "您的产品线为：",self.productLine,"请把信息反馈到技术人员，检查推送策略是否正确")
            print(f"{Colors.FAIL}")


if __name__ == "__main__":
    checkObject = check()
    checkObject.getServerAddress()
    checkObject.getPcInfo()
    checkObject.getSystemVersion()
    checkObject.getProductLine()
    checkObject.checkPushResult()
EOF
    CONFIG_DIR="/usr/share/kylin-update-desktop-config/config"
    append_desc=""
    if ! [ -f "$SOURCES_LIST" ]; then
        error_desc="系统更新源不存在:$SOURCES_LIST"
        solution="请检查1、是否导入序列号 2、更新管理平台配置的更新策略是否正确 3、若配置没问题请联系源管理研发。"
        append_desc=$(python3 /tmp/check-sources.py 2>&1)
    elif [ -f $PUSH_CONTENT ]; then
        # 检查内容是否为空
        if [ -z "$(tr -d '[:space:]' < $PUSH_CONTENT)" ]; then
            error_desc="更新推送内容为空。（/var/lib/kylin-software-properties/template/important.list）"
            solution="请检查1、是否导入序列号 2、更新管理平台配置的更新策略是否正确 3、若配置没问题请联系源管理研发。"
            append_desc=$(python3 /tmp/check-sources.py 2>&1)
        else
            append_other=$(cat $PUSH_CONTENT)
            read -r -a items <<< "$(cat $PUSH_CONTENT)"

            for package_name in "${items[@]}"; do
                # 检查包是否在 apt 缓存中
                if apt-cache show "$package_name" > /dev/null 2>&1; then
                    ff="ff"
                else
                    conf_file="$CONFIG_DIR/$package_name.json"
                    if ! [ -f $conf_file ]; then
                        error_desc="kylin-update-desktop-config包中没有'$package_name'推送的分组配置。"
                        package_version=$(apt-cache show "kylin-update-desktop-config" | grep '^Version:' | awk '{print $2}')
                        solution="请联系版本组检查config配置或检查推送的源是否正确，当前版本kylin-update-desktop-config=$package_version"
                        arg_prompt "$error_desc" "$solution" "$append_desc"
                    fi
                fi
            done
            fn_update_problem
        fi
    elif ! [ -f $PUSH_CONTENT ]; then
        error_desc="没有推送内容文件。"
        solution="请检查是否导入序列号$append_other"
    else
        fn_update_problem
    fi
    arg_prompt "$error_desc" "$solution" "$append_desc"
}


fn_check_download_pkgs() {
    search_string="Hashes of received file"
    match_line=$(grep -n "$search_string" "$UPDATER_LOG_PATH" | head -n 1 | cut -d ":" -f 1)

    # 如果找到匹配的行
    if [ -n "$match_line" ]; then
        # 输出匹配内容上下10行的内容
        echo "发现下面错误信息:"
        echo ""
        tail -n +$((match_line - 8)) "$UPDATER_LOG_PATH" | head -n 17

        echo "请联系版本组检查离线源或者在线源内容"
    else
        echo "当前使用在线源更新方式，测试下载软件包从源中"
        fn_apt_download kylin-system-updater
        if [ $? -eq 0 ]; then
            rm *kylin-system-updater*.deb
            echo "下载软件包正常，导出系统更新日志到禅道"
        else
            if ! [ "$OFFLINE_SOURCE_FLAG" = "$TRUE" ]; then
                echo "测试下载失败，检查网络连接，网络连接完善后使用apt download tree 软件包测试是否正常"
            else
                echo "测试下载失败，联系版本组检查离线源是否正常"
            fi
        fi
    fi
}

fn_check_source_manager() {
    info "${BOLD}进行刷新源测试，最长时间30s请稍后...${NO_COLOR}"
    error_desc="源管理客户端刷新软件源与推送内容失败。"
    solution="检查设备网络是否正常，无法解决时请联系源管理研发人员。"
    # local append_desc=$(timeout 20s kylin-source-update 2>&1)

    # if [ $? -eq 0 ]; then
    #     if [[ $append_desc == *"true"* ]]; then
    #         fn_update_problem
    #     else
    #         error_desc="源管理客户端刷新源失败"
    #         solution="检查设备网络是否正常，无法解决时请联系源管理研发人员。"
    #     fi
    # elif [ $? -eq 124 ]; then
    #     error_desc="源管理客户端刷新源超时。"
    #     solution="1、请检查网络是否正常 2、若在公司hw护网期间，多尝试几次点击检查更新是否可行，无法解决时请联系源管理研发人员。"
    # else
    #     fn_update_problem
    # fi
    arg_prompt "$error_desc" "$solution" "$append_desc"
}

fn_check_config_depends() {
    error_desc="kylin-update-desktop-config存在依赖冲突无法进行升级。"
    upgrade_pkgs="kylin-update-desktop-config"
    simulate_desc=$(LANG=zh_CN.UTF-8 $APT_RUN install $upgrade_pkgs --simulate 2>&1)
    echo "$upgrade_pkgs" > $ANALYSE_LOG_DIR/upgrade_pkgs.list
    # 检查字符串是否存在
    if [[ $simulate_desc == *"下列软件包有未满足的依赖关系"* ]]; then
        solution="联系版本组解决相关依赖问题。 解决下面存在的依赖冲突问题。"
        append_desc=$(echo "$simulate_desc" | sed -n '/下列软件包有未满足的依赖关系：/,$p')
    else
        solution="请联系系统更新研发，并上传日志。"
        echo "$simulate_desc" > $ANALYSE_LOG_DIR/simulate-install.log
    fi
    arg_prompt "$error_desc" "$solution" "$append_desc"
}

fn_check_depends_problem() {
    error_desc="更新内容存在依赖冲突无法进行升级。"
    pkgs_list_file="/tmp/upgrade_pkgs.list"
    depends_problem_flag="Resolver calculation Packages List:"

    upgrade_pkgs=$(tac $UPDATER_LOG_PATH | grep -m 1 "$depends_problem_flag" | awk -F "$depends_problem_flag" '{print $2}')

    if [ -z "$upgrade_pkgs" ]; then
        fn_update_problem
    fi

    simulate_desc=$(LANG=zh_CN.UTF-8 $APT_RUN install $upgrade_pkgs --simulate 2>&1)

    echo "$upgrade_pkgs" > $ANALYSE_LOG_DIR/upgrade_pkgs.list
    # 检查字符串是否存在
    if [[ $simulate_desc == *"下列软件包有未满足的依赖关系"* ]]; then
        if [ "$INTERNAL_TRIAL" = "$TRUE" ]; then
            solution="内部试用：1、查看下面其他描述提示的依赖冲突自行降级修复依赖关系，执行apt -c /etc/kylin-system-updater/apt_private.conf install 软件包（需要调整的包）；2、修复完成后检查是否修复成功执行$APT_RUN install 软件包（$ANALYSE_LOG_DIR/upgrade_pkgs.list内容） 3、不报依赖问题再打开设置-更新进行系统更新"
        else
            solution="联系版本组解决相关依赖问题。 将要升级的软件包列表查看:$pkgs_list_file ；或可执行apt -c /etc/kylin-system-updater/apt_private.conf install 文件内的软件包 --simualte"
            echo "$upgrade_pkgs" > $pkgs_list_file
        fi
        
        append_desc=$(echo "$simulate_desc" | sed -n '/下列软件包有未满足的依赖关系：/,$p')
    else
        solution="请联系系统更新研发，并上传日志。"
        echo "$simulate_desc" > $ANALYSE_LOG_DIR/simulate-install.log
    fi

    report_apt_lists
    arg_prompt "$error_desc" "$solution" "$append_desc"
}

fn_self_depends() {
    # 获取当前安装的 kylin-video 版本
    installed_version=$(dpkg-query -W -f='${Version}' kylin-system-updater)

    # 要比较的版本
    required_version="2.0.5.16-0k7.62"

    # 比较版本
    if dpkg --compare-versions "$installed_version" gt "$required_version"; then
        fn_check_depends_problem
    else
        upgrade_pkgs="kylin-system-updater kylin-update-frontend aptdaemon kylin-background-upgrade kylin-software-properties"
        simulate_desc=$(LANG=zh_CN.UTF-8 $APT_RUN install $upgrade_pkgs --simulate 2>&1)
        echo "$upgrade_pkgs" > $ANALYSE_LOG_DIR/upgrade_pkgs.list
        # 检查字符串是否存在
        if [[ $simulate_desc == *"下列软件包有未满足的依赖关系"* ]]; then
            error_desc="系统更新进行自升级时，缺少依赖无法进行升级。"
            solution="联系版本组解决相关依赖问题。 解决下面存在的依赖冲突问题"
            append_desc=$(echo "$simulate_desc" | sed -n '/下列软件包有未满足的依赖关系：/,$p')
        else
            error_desc="系统更新进行自升级缺少依赖但无法审计缺少那些依赖。"
            solution="请联系系统更新研发，并上传日志。"
            echo "$simulate_desc" > $ANALYSE_LOG_DIR/simulate-install.log
        fi
    fi
    arg_prompt "$error_desc" "$solution" "$append_desc"
}


fn_check_version() {
    version_check_flag="Version Check Desc:"
    customized_path_flag="Customized patch: "
    customized_other_flag="Customized: "
    intermediate_path_flag="Version mismatch "

    error_desc=$(tac $UPDATER_LOG_PATH | grep -m 1 "$version_check_flag" | awk -F "$version_check_flag" '{print $2}')
    solution="联系版本组校对版本与补丁匹配数据库，并提交相应bug"

    if [ "$INTERNAL_TRIAL" = "$TRUE" ]; then
        solution="内部试用：1、识别为定制版本系统时，查看其他描述提示的定制包 换成非定制项目的版本可以解决 2、无法解决时将导出的日志提交系统更新研发。"
    else
        solution="1、非测试自己推送的源，则联系版本组校对版本与补丁匹配数据库，并提交相应bug；2、若测试内部自己搭建的推送平台并推送源，则直接删除/usr/share/kylin-update-desktop-config/version/ 目录跳过检查。"
    fi

    append_desc=$(tac $UPDATER_LOG_PATH | grep -E -m 1 "$intermediate_path_flag|$customized_path_flag|$customized_other_flag")
    if [[ $append_desc == *$customized_path_flag* ]]; then
        append_desc="定制源：$(echo $append_desc | awk -F "$customized_path_flag" '{print $2}')"
    elif [[ $append_desc == *$intermediate_path_flag* ]]; then
        append_desc="不在包版本数据库的包列表：$(echo $append_desc | awk -F "$intermediate_path_flag" '{print $2}')"
    elif [[ $append_desc == *$customized_other_flag* ]]; then
        append_desc="定制版本的包信息：$(echo $append_desc | awk -F "$customized_other_flag" '{print $2}')"
    else
        append_desc="无法识别详细错误，查看系统更新日志"
    fi
    arg_prompt "$error_desc" "$solution" "$append_desc"
}

fn_check_remove_essential() {

    version_check_flag="Essential Packages"
    delete_flag="DELETE_DETAILS"
    error_desc=$(tac $UPDATER_LOG_PATH | grep -m 1 "$version_check_flag" | awk -F "$version_check_flag" '{print $2}')
    error_desc="被删除的重要软件包：$error_desc"
    solution="联系版本组检查此包是否可以被卸载，若可以删除则在config包install_list列表中删除此包，删除原因查看系统更新日志，检索DELETE_DETAILS字符。"
    
    append_desc=$(tac $UPDATER_LOG_PATH | grep -m 5 "$delete_flag" | awk -F "$delete_flag" '{print $2}')
    append_desc="被软件包原因：$append_desc"
    arg_prompt "$error_desc" "$solution" "$append_desc"
}

fn_check_program_execption() {
    check_flag="PermissionError:"

    error_desc=$(tac $UPDATER_LOG_PATH | grep -m 1 "$check_flag")

    if [ -z "$error_desc" ]; then
        error_desc="安全功能导致系统更新没有权限写入内容文件。"
        solution="将安全功能关闭后再进行系统更新，执行setstatus disable 关闭安全控制"
    else
        fn_update_problem
    fi

    arg_prompt "$error_desc" "$solution" "$append_desc"
}
fn_check_source_lock() {
    # 文件列表
    FILES=(
        "/etc/apt/sources.list"
        "/var/lib/kylin-system-updater/sources.list"
        "/var/lib/kylin-software-properties/template/important.list"
    )

    error_desc="系统更新必要文件被加锁，禁止写入。"
    solution="将下面被锁文件进行解锁，执行sudo chattr -i 文件。"
    # 遍历文件列表
    for FILE in "${FILES[@]}"; do
        # 检查文件是否存在
        if [[ -e "$FILE" ]]; then
            # 获取文件的属性
            ATTR=$(lsattr "$FILE" 2>/dev/null)

            # 检查是否包含 'i' 标志
            if [[ "$ATTR" =~ -i- ]]; then
                append_desc="被锁文件:$FILE "
                arg_prompt "$error_desc" "$solution" "$append_desc"
            fi
        fi
    done
}


fn_download_deb_failed() {
    msg=$(tac "$UPDATER_LOG_PATH" | grep -m 2 'Failed to fetch')
    error_desc="软件包下载失败。"
    solution="检查网络是否正常，若正常测试其他设备是否也有相同问题，都存在问题时联系版本组或者平台组检查源"

    arg_prompt "$error_desc" "$solution" "$msg"
}

fn_check_remove_pkg() {

    delete_check_flag="UpdateDependResloveStatus:resolver_status = True , remove_status = True , remove_pkgs ="
    delete_flag="DELETE_DETAILS"

    error_desc=$(tac $UPDATER_LOG_PATH | grep -m 1 "$delete_check_flag" | grep -o 'remove_pkgs = [^ ]*,')
    error_desc="被删除的软件包：$error_desc"
    solution="联系版本组检查此包是否可以被卸载，若可以删除则在config包remove_white_list列表中删除此包，删除原因查看系统更新日志，检索DELETE_DETAILS字符。"
    
    append_desc=$(tac $UPDATER_LOG_PATH | grep -m 5 "$delete_flag" | awk -F "$delete_flag" '{print $2}')
    append_desc="被软件包原因：$append_desc"
    arg_prompt "$error_desc" "$solution" "$append_desc"
}

fn_libos_problem() {
    error_desc="系统关键库不匹配存在异常，/usr/local/lib/下包含非法自安装的so库。"

    if [ "$INTERNAL_TRIAL" = "$TRUE" ]; then
        solution="内部试用：把/usr/local/lib/下的so库手动移动到其他位置，再进行更新"
    else
        solution="测试：请联系系统更新研发，并上传日志。"
    fi

    check_flag="Check: illegal library:"

    append_desc=$(tac $UPDATER_LOG_PATH | grep -m 5 "$check_flag" | awk -F "$check_flag" '{print $2}')
    append_desc="不满足的so库：$append_desc"

    arg_prompt "$error_desc" "$solution" "$append_desc"
}

fn_check_broken_pkg() {
    error_desc="系统存在无法自动修复破损软件包，需要进行手动修复。"
    solution="手动执行命令：$APT_RUN install -f 修复系统存在破损的软件包。但注意卸载的包不要将系统卸载奔溃了。"
    arg_prompt "$error_desc" "$solution"
}

fn_check_network() {
    error_desc="网络检查失败。"
    solution="请检查是否正常连接网络"
    arg_prompt "$error_desc" "$solution"
}

fn_check_write_sources() {
    error_desc="修改系统更新源文件权限。导致无法写入内容。"
    solution="执行chattr -i /var/lib/kylin-system-updater/sources.list 解除文件锁。"
    arg_prompt "$error_desc" "$solution"
}

fn_update_lock_problem() {
    error_desc="其他应用程序正在使用apt/dpkg来进行装包操作。"

    pid=$(sudo lsof /var/lib/dpkg/lock-frontend /var/lib/dpkg/lock | awk 'NR>1 {print $2}' | head -n 1)
    if [ -z "$pid" ]; then
        fn_update_problem
    else
        process_name=$(ps -p $pid -o comm=)
        solution="1、请检查是否打开终端使用apt/dpkg来安装软件包，2、未使用时联系系统更新研发处理。"
        append_desc="进程 $process_name (PID: $pid) 正在占用 apt"
    fi
    arg_prompt "$error_desc" "$solution" "$append_desc"
}

fn_time_compare() {
    input_date=$1

    # 获取当前年份
    current_year=$(date +%Y)

    # 把给定的日期转换为包含当前年份的日期格式
    input_date_full="$current_year-$input_date"

    # 获取当前日期和两天前的日期（使用日期格式 %Y-%m-%d）
    current_date=$(date +%Y-%m-%d)
    two_days_ago=$(date -d "-10 days" +%Y-%m-%d)

    # 转换日期为秒数以便比较
    input_date_seconds=$(date -d "$input_date_full" +%s)
    current_date_seconds=$(date -d "$current_date" +%s)
    two_days_ago_seconds=$(date -d "$two_days_ago" +%s)

    # 比较日期
    if [[ "$input_date_seconds" -ge "$two_days_ago_seconds" && "$input_date_seconds" -le "$current_date_seconds" ]]; then
        local ttt=tt
    else
        fn_update_problem
    fi
}

main() {
    while :; do
        case $1 in
            -h|-\?|--help)
                fn_display_usage
                exit
                ;;
            -f|--install-failed)
                shift
                fn_install_failed details
                exit 0
                ;;
            -f|--install-failed-easy)
                shift
                fn_install_failed easy 
                exit 0
                ;;
            *)
                break
        esac

        shift
    done

    # 提取日志最后的错误信息进行分析
    bug_msg=$(tac "$UPDATER_LOG_PATH" | grep -m 1 'UpdateDependResloveStatus:resolver_status = False\|UpdateDependResloveStatus:resolver_status = True , remove_status = True\|UpdateDetectFinished success = False\|UpdateDetectFinished success = dbus.Boolean(False)\|UpdateDownloadFinished success = False\|UpdateInstallFinished success = False\|ForceUpdatePopup success = True\|等待其他软件管理程序退出\|UpdateDetectFinished success = True , upgrade_group = \[\]')

    ERROR_DESC=$(echo "$bug_msg"  | grep -o 'error_desc = [^ ]*' | sed 's/error_desc = //')
    ERROR_STR=$(echo "$bug_msg"  | grep -o 'error_string = [^ ]*' | sed 's/error_string = //')

    if ! [ -z "$ERROR_DESC" ]; then
        # 存在依赖冲突 进行了全盘更新 升级完成后再次更新报错
        if [[ $bug_msg == *"UpdateDownloadFinished success = False , upgrade_group = ['dist-upgrade'], error_string = 更新包存在异常！ , error_desc = 本次更新无法检测到可升级的软件包。"* ]]; then
            ERROR_DESC="更新异常弹窗，提示全盘更新。"
        else
            pass=pass
        fi
    elif [[ $bug_msg == *"UpdateDetectFinished success = True , upgrade_group = []"* ]]; then
        ERROR_DESC="您的系统已是最新"
    elif [[ $bug_msg == *"ForceUpdatePopup success = True"* ]]; then
        ERROR_DESC="版本与补丁校验不匹配。"
    elif [[ $bug_msg == *"UpdateDependResloveStatus:resolver_status = False"* ]]; then
        ERROR_DESC="更新异常弹窗，提示全盘更新。"
    elif [[ $bug_msg == *"UpdateDependResloveStatus:resolver_status = True , remove_status = True"* ]]; then
        ERROR_DESC="依赖冲突，需要卸载软件包。"
    elif [[ $bug_msg == *"等待其他软件管理程序退出"* ]]; then
        ERROR_DESC="其他软件管理程序（apt/dpkg）占用安装。"
    else
        fn_update_problem
    fi
    
    case $ERROR_DESC in
        无法安装或移除软件包。|安装或移除软件包失败。|软件包操作异常|软件包操作异常无法安装或移除软件包。|The)
            ERROR_DESC="$ERROR_DESC错误码：#0200"
            fn_install_failed
            ;;
        无法访问源管理服务器，请稍后再试|推送服务器连接异常)
            ERROR_DESC="$ERROR_DESC错误码：#0100"
            fn_check_source_manager
            ;;
        无法下载软件仓库信息，检查您的网络连接。|检查您的网络连接。|无法下载软件仓库信息)
            case $ERROR_STR in
                无法下载软件包文件|下载软件包文件失败|\#0203)
                    ERROR_DESC="$ERROR_DESC错误码：#0203"
                    fn_download_deb_failed
                    ;;
                无法下载软件仓库信息|检查更新异常！|下载软件仓库信息失败，检查您的网络连接。|\#0115)
                    ERROR_DESC="$ERROR_DESC错误码：#0115"
                    fn_check_update_source
                    ;;
                *)
                    fn_update_problem
            esac
            ;;
        您的系统已是最新)
            ERROR_DESC="$ERROR_DESC(需要进行系统更新但一直提示最新情况)"
            fn_check_latest
            ;;
        请检查您的网络连接后再试。)
            fn_check_network
            ;;
        版本与补丁校验不匹配。)
            ERROR_DESC="$ERROR_DESC错误码：#0117"
            fn_check_version
            ;;
        \#0202)
            ERROR_DESC="您要求删除一个系统必要的软件包。错误码：#0202"
            fn_check_remove_essential
            ;;
        依赖冲突，需要卸载软件包。)
            fn_check_remove_pkg
            ;;
        更新异常弹窗，提示全盘更新。|\#0001)
            ERROR_DESC="更新异常弹窗，提示全盘更新。"
            fn_check_depends_problem
            ;;
        这是个很严重的问题。请稍后重试。如果问题还出现，请向开发人员报告一个错误。)
            fn_check_update_source
            ;;
        无法进行优先升级，请检查优先升级包相关依赖。)
            ERROR_DESC="$ERROR_DESC错误码：#0112"
            fn_self_depends
            ;;
        修复系统APT环境异常，请检查系统APT环境。)
            ERROR_DESC="$ERROR_DESC错误码：#0114"
            fn_check_broken_pkg
            ;;
        无法安装组配置包，请检查组配置包相关依赖。)
            ERROR_DESC="$ERROR_DESC错误码：#0111"
            fn_check_config_depends
            ;;
        系统关键库不匹配。)
            fn_libos_problem
            ;;
        Write)
            ERROR_DESC="没有写入系统更新源权限。错误码：#0122"
            fn_check_write_sources
            ;;
        本次更新无法检测到可升级的软件包。)
            fn_update_problem
            ;;
        其他软件管理程序（apt/dpkg）占用安装。)
            fn_update_lock_problem
            ;;
        程序执行异常，请联系管理员解决。)
            fn_check_program_execption
            ;;
        *)
            fn_update_problem
    esac
}

main "$@"

